#include "GY80.h"

// コンストラクタ ===================================================
GY80::GY80(char modeflag): umode(modeflag){
	/*下位4ビットから各センサーを使用するかどうか確認
	| 4bit | 3bit | 2bit | 1bit     |
	| 気圧 | 磁気 | 加速 | ジャイロ |
	
	例)
	全部(デフォルト) = 1111 (0x0F)
	気圧,加速,ジャイロ = 1011 (0x0B)
	磁気,加速 = 0110 (0x06)
	気圧 = 1000 (0x08)
	*/
}

// I2Cとセンサーの初期化開始 ========================================
void GY80::init(bool i2cinit){
	if(i2cinit){
		IIC_CHANNEL_NAME.setFrequency(IIC_SIGNALFREQ_HZ);
		IIC_CHANNEL_NAME.begin();
	}

	if(umode & 0x01) initGyro();
	if(umode & 0x02) initAccelero();
	if(umode & 0x04) initCompass();
	if(umode & 0x08) initAPressure();
}

// I2C読み取り ===================================
// ＠単純読み取り
unsigned char GY80::wire_read(unsigned char devAddr, unsigned char reg){
	IIC_CHANNEL_NAME.beginTransmission(devAddr);
	IIC_CHANNEL_NAME.write(reg);
	IIC_CHANNEL_NAME.endTransmission();
	IIC_CHANNEL_NAME.requestFrom(devAddr, 1U);
	return IIC_CHANNEL_NAME.read();
}
// ＠複数読み取り
void GY80::wire_readArray(unsigned char *buff, unsigned char devAddr, unsigned char reg, unsigned char length){
	IIC_CHANNEL_NAME.beginTransmission(devAddr);
	IIC_CHANNEL_NAME.write(reg);
	IIC_CHANNEL_NAME.endTransmission();
	IIC_CHANNEL_NAME.requestFrom(devAddr, length);
	unsigned char i = 0;
	while(i<length && IIC_CHANNEL_NAME.available()){
		buff[i++] = IIC_CHANNEL_NAME.read();
	}
}

// I2C書き込み ===================================
void GY80::wire_write(unsigned char devAddr, unsigned char reg, unsigned char val){
	IIC_CHANNEL_NAME.beginTransmission(devAddr);
	IIC_CHANNEL_NAME.write(reg);
	IIC_CHANNEL_NAME.write(val);
	IIC_CHANNEL_NAME.endTransmission();
}

// ジャイロセンサー初期化 ===========================================
void GY80::initGyro(){
	wire_read(IIC_ADDR_GYRO, GYRO_CTRL_REG1); //dummy(最初のほうの信号だけ妙に安定しないため)
	wire_read(IIC_ADDR_GYRO, GYRO_CTRL_REG2); //dummy(最初のほうの信号だけ妙に安定しないため)
	wire_read(IIC_ADDR_GYRO, GYRO_CTRL_REG3); //dummy(最初のほうの信号だけ妙に安定しないため)
	delay(50);
	
	wire_write(IIC_ADDR_GYRO, GYRO_CTRL_REG1, 0x5F); //datarate:200Hz,bandwidth:25Hz,power on,XYZenable
	wire_write(IIC_ADDR_GYRO, GYRO_CTRL_REG2, 0x01); //High pass filter 8Hz
	wire_write(IIC_ADDR_GYRO, GYRO_CTRL_REG3, 0x80); //Interrupt enbale INT1 pin
	wire_write(IIC_ADDR_GYRO, GYRO_CTRL_REG4, 0x10); //sensor 500dps
	
	delay(300);
}

// ジャイロセンサーデータ取得 =======================================
// ＠全軸+温度
void GY80::getGyroAll(bool isDEG){
	short x, y, z;
	x = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_L);
	y = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_L);
	z = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_L);
	g.temp = wire_read(IIC_ADDR_GYRO, GYRO_OUT_TEMP);
	g.x = (isDEG)? x * GYRO_SCALE : radians(x * GYRO_SCALE);
	g.y = (isDEG)? y * GYRO_SCALE : radians(y * GYRO_SCALE);
	g.z = (isDEG)? z * GYRO_SCALE : radians(z * GYRO_SCALE);
}
// ＠全軸
void GY80::getGyroAxis(bool isDEG){
	short x, y, z;
	x = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_L);
	y = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_L);
	z = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_L);
	g.x = (isDEG)? x * GYRO_SCALE : radians(x * GYRO_SCALE);
	g.y = (isDEG)? y * GYRO_SCALE : radians(y * GYRO_SCALE);
	g.z = (isDEG)? z * GYRO_SCALE : radians(z * GYRO_SCALE);
}
// ＠XY軸
void GY80::getGyroAxisXY(bool isDEG){
	short x, y;
	x = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_L);
	y = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_L);
	g.x = (isDEG)? x * GYRO_SCALE : radians(x * GYRO_SCALE);
	g.y = (isDEG)? y * GYRO_SCALE : radians(y * GYRO_SCALE);
}
// ＠XZ軸
void GY80::getGyroAxisXZ(bool isDEG){
	short x, z;
	x = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_L);
	z = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_L);
	g.x = (isDEG)? x * GYRO_SCALE : radians(x * GYRO_SCALE);
	g.z = (isDEG)? z * GYRO_SCALE : radians(z * GYRO_SCALE);
}
// ＠YZ軸
void GY80::getGyroAxisYZ(bool isDEG){
	short y, z;
	y = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_L);
	z = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_L);
	g.y = (isDEG)? y * GYRO_SCALE : radians(y * GYRO_SCALE);
	g.z = (isDEG)? z * GYRO_SCALE : radians(z * GYRO_SCALE);
}
// ＠X軸
void GY80::getGyroAxisX(bool isDEG){
	short x;
	x = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_L);
	g.x = (isDEG)? x * GYRO_SCALE : radians(x * GYRO_SCALE);
}
// ＠Y軸
void GY80::getGyroAxisY(bool isDEG){
	short y;
	y = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_L);
	g.y = (isDEG)? y * GYRO_SCALE : radians(y * GYRO_SCALE);
}
// ＠Z軸
void GY80::getGyroAxisZ(bool isDEG){
	short z;
	z = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_L);
	g.z = (isDEG)? z * GYRO_SCALE : radians(z * GYRO_SCALE);
}
// ＠温度
void GY80::getGyroTemp(){
	g.temp = wire_read(IIC_ADDR_GYRO, GYRO_OUT_TEMP);
}

// 加速度センサー初期化 =============================================
void GY80::initAccelero(){
	wire_read(IIC_ADDR_ACCEL, ACCEL_POWER_CTL); //dummy(最初のほうの信号だけ妙に安定しないため)
	wire_read(IIC_ADDR_ACCEL, ACCEL_INT_ENABLE); //dummy(最初のほうの信号だけ妙に安定しないため)
	wire_read(IIC_ADDR_ACCEL, ACCEL_DATA_FORMAT); //dummy(最初のほうの信号だけ妙に安定しないため)
	delay(50);
	
	wire_write(IIC_ADDR_ACCEL, ACCEL_THRESH_FF, 0x09); //Freefall 600mg
	wire_write(IIC_ADDR_ACCEL, ACCEL_TIME_FF, 0x46); //Freefall 350ms
	wire_write(IIC_ADDR_ACCEL, ACCEL_TAP_AXES, 0x07); //TAP xyz disable
	wire_write(IIC_ADDR_ACCEL, ACCEL_BW_RATE, 0x0B); //data rate 200Hz, Bandwidth 100Hz power-savemode OFF
	wire_write(IIC_ADDR_ACCEL, ACCEL_POWER_CTL, 0x08); //Sleep OFF, Measure ON
	wire_write(IIC_ADDR_ACCEL, ACCEL_INT_ENABLE, 0x04); //Freefall interrupt INT1 pin ON
	wire_write(IIC_ADDR_ACCEL, ACCEL_DATA_FORMAT, 0x0B); //Full resolution, Range +-16g 13bit
	
	delay(300);
}

// 加速度センサー補正 ===============================================
// ＠自動平均補正
void GY80::setAcceleroOffsetAuto(char sample_n){
	//N個のサンプルから平均値を取得し、補正値としてレジスタに書き込む
	sample_n = (sample_n > 100)? 100 : (sample_n < 10)? 10 : sample_n;
	float xf=0.0, yf=0.0, zf=0.0;
	short x=0, y=0, z=0;
	unsigned char ary[6];
	for(char n=0; n<sample_n; n++){
		wire_readArray(ary, IIC_ADDR_ACCEL, ACCEL_DATAX0, 6);
		x = (ary[1] << 8) | ary[0];
		y = (ary[3] << 8) | ary[2];
		z = (ary[5] << 8) | ary[4];
		xf += (x * ACCEL_SCALE);
		yf += (y * ACCEL_SCALE);
		zf += (z * ACCEL_SCALE);
		delay(10);
	}
	ac.offX = -1.0 * (xf / sample_n);
	ac.offY = -1.0 * (yf / sample_n);
	ac.offZ = -1.0 * (zf / sample_n) + 1.0; //Z軸は重力加速度の影響を受ける
	//補正値を書き込み
	setAcceleroOffset(ac.offX, ac.offY, ac.offZ);
}
// ＠自動平均補正(対象軸を指定)
void GY80::setAcceleroOffsetAuto(bool targetX, bool targetY, bool targetZ, char sample_n){
	//N個のサンプルから平均値を取得し、補正値としてレジスタに書き込む
	sample_n = (sample_n > 100)? 100 : (sample_n < 10)? 10 : sample_n;
	float xf=0.0, yf=0.0, zf=0.0;
	short x=0, y=0, z=0;
	unsigned char ary[6];
	for(char n=0; n<sample_n; n++){
		wire_readArray(ary, IIC_ADDR_ACCEL, ACCEL_DATAX0, 6);
		x = (ary[1] << 8) | ary[0];
		y = (ary[3] << 8) | ary[2];
		z = (ary[5] << 8) | ary[4];
		xf += (x * ACCEL_SCALE);
		yf += (y * ACCEL_SCALE);
		zf += (z * ACCEL_SCALE);
		delay(10);
	}
	//各軸が対象であれば構造体の補正値を更新
	if(targetX) ac.offX = -1.0 * (xf / sample_n);
	if(targetY) ac.offY = -1.0 * (yf / sample_n);
	if(targetZ) ac.offZ = -1.0 * (zf / sample_n) + 1.0; //Z軸は重力加速度の影響を受ける
	//補正値を書き込み
	setAcceleroOffset(ac.offX, ac.offY, ac.offZ);
}
// ＠数値指定補正用
void GY80::setAcceleroOffset(float offsetX, float offsetY, float offsetZ){
	//0xFF(-2g)から0x7F(+2g)の間で符号付き8ビットで補正
	//+1.25gの補正をしたい場合、1.25/0.015625 = 80(0x50)になる
	if(offsetX != 0.0f) wire_write(IIC_ADDR_ACCEL, ACCEL_OFSX, (char)(offsetX / 0.015625));
	if(offsetY != 0.0f) wire_write(IIC_ADDR_ACCEL, ACCEL_OFSY, (char)(offsetY / 0.015625));
	if(offsetZ != 0.0f) wire_write(IIC_ADDR_ACCEL, ACCEL_OFSZ, (char)(offsetZ / 0.015625));
	//構造体の中の変数も更新
	getAcceleroOffset();
}
// 加速度センサーデータ取得 =========================================
// ＠全軸+補正値
void GY80::getAcceleroAll(){
	unsigned char ary[6];
	wire_readArray(ary, IIC_ADDR_ACCEL, ACCEL_DATAX0, 6);
	short x, y, z;
	x = (ary[1] << 8) | ary[0];
	y = (ary[3] << 8) | ary[2];
	z = (ary[5] << 8) | ary[4];
	ac.x = x * ACCEL_SCALE;
	ac.y = y * ACCEL_SCALE;
	ac.z = z * ACCEL_SCALE;
	ac.offX = 0.015625 * (signed char)wire_read(IIC_ADDR_ACCEL, ACCEL_OFSX);
	ac.offY = 0.015625 * (signed char)wire_read(IIC_ADDR_ACCEL, ACCEL_OFSY);
	ac.offZ = 0.015625 * (signed char)wire_read(IIC_ADDR_ACCEL, ACCEL_OFSZ);
}
// ＠全軸
void GY80::getAcceleroAxis(){
	unsigned char ary[6];
	wire_readArray(ary, IIC_ADDR_ACCEL, ACCEL_DATAX0, 6);
	short x, y, z;
	x = (ary[1] << 8) | ary[0];
	y = (ary[3] << 8) | ary[2];
	z = (ary[5] << 8) | ary[4];
	ac.x = x * ACCEL_SCALE;
	ac.y = y * ACCEL_SCALE;
	ac.z = z * ACCEL_SCALE;
}
// ＠XY軸
void GY80::getAcceleroAxisXY(){
	unsigned char ary[6];
	wire_readArray(ary, IIC_ADDR_ACCEL, ACCEL_DATAX0, 6);
	short x, y;
	x = (ary[1] << 8) | ary[0];
	y = (ary[3] << 8) | ary[2];
	ac.x = x * ACCEL_SCALE;
	ac.y = y * ACCEL_SCALE;
}
// ＠XZ軸
void GY80::getAcceleroAxisXZ(){
	unsigned char ary[6];
	wire_readArray(ary, IIC_ADDR_ACCEL, ACCEL_DATAX0, 6);
	short x, z;
	x = (ary[1] << 8) | ary[0];
	z = (ary[5] << 8) | ary[4];
	ac.x = x * ACCEL_SCALE;
	ac.z = z * ACCEL_SCALE;
}
// ＠YZ軸
void GY80::getAcceleroAxisYZ(){
	unsigned char ary[6];
	wire_readArray(ary, IIC_ADDR_ACCEL, ACCEL_DATAX0, 6);
	short y, z;
	y = (ary[3] << 8) | ary[2];
	z = (ary[5] << 8) | ary[4];
	ac.y = y * ACCEL_SCALE;
	ac.z = z * ACCEL_SCALE;
}
// ＠X軸
void GY80::getAcceleroAxisX(){
	unsigned char ary[6];
	wire_readArray(ary, IIC_ADDR_ACCEL, ACCEL_DATAX0, 6);
	short x = (ary[1] << 8) | ary[0];
	ac.x = x * ACCEL_SCALE;
}
// ＠Y軸
void GY80::getAcceleroAxisY(){
	unsigned char ary[6];
	wire_readArray(ary, IIC_ADDR_ACCEL, ACCEL_DATAX0, 6);
	short y = (ary[3] << 8) | ary[2];
	ac.y = y * ACCEL_SCALE;
}
// ＠Z軸
void GY80::getAcceleroAxisZ(){
	unsigned char ary[6];
	wire_readArray(ary, IIC_ADDR_ACCEL, ACCEL_DATAX0, 6);
	short z = (ary[5] << 8) | ary[4];
	ac.z = z * ACCEL_SCALE;
}
// ＠補正値取得
void GY80::getAcceleroOffset(){
	ac.offX = 0.015625 * (signed char)wire_read(IIC_ADDR_ACCEL, ACCEL_OFSX);
	ac.offY = 0.015625 * (signed char)wire_read(IIC_ADDR_ACCEL, ACCEL_OFSY);
	ac.offZ = 0.015625 * (signed char)wire_read(IIC_ADDR_ACCEL, ACCEL_OFSZ);
}

// 地磁気センサー初期化 ==============================================
void GY80::initCompass(){
	wire_read(IIC_ADDR_CMPS, CMPS_REGA); //dummy(最初のほうの信号だけ妙に安定しないため)
	wire_read(IIC_ADDR_CMPS, CMPS_REGB); //dummy(最初のほうの信号だけ妙に安定しないため)
	wire_read(IIC_ADDR_CMPS, CMPS_MODE); //dummy(最初のほうの信号だけ妙に安定しないため)
	delay(50);
	
	wire_write(IIC_ADDR_CMPS, CMPS_REGA, 0x58); //out 4samples average, data rate 75Hz, Normal measure
	wire_write(IIC_ADDR_CMPS, CMPS_REGB, 0x40); //Gain 820 (+-1.9Ga range)
	wire_write(IIC_ADDR_CMPS, CMPS_MODE, 0x00); //Highspeed i2c(3400kHz) OFF, Continuous measurement mode
	
	delay(300);
}

// 地磁気センサーデータ取得 ==========================================
// ＠全軸
void GY80::getCompassAll(){
	unsigned char ary[6];
	wire_readArray(ary, IIC_ADDR_CMPS, CMPS_DATA_XMSB, 6);
	short x, y, z;
	x = (ary[0] << 8) | ary[1];
	z = (ary[2] << 8) | ary[3]; //レジスタの並びがX, Z, Yであることに注意
	y = (ary[4] << 8) | ary[5];
	c.x = x * CMPS_SCALE;
	c.y = y * CMPS_SCALE;
	c.z = z * CMPS_SCALE;
}
// ＠全軸(getCompassAllとgetCompassAxisは同一)
void GY80::getCompassAxis(){
	unsigned char ary[6];
	wire_readArray(ary, IIC_ADDR_CMPS, CMPS_DATA_XMSB, 6);
	short x, y, z;
	x = (ary[0] << 8) | ary[1];
	z = (ary[2] << 8) | ary[3]; //レジスタの並びがX, Z, Yであることに注意
	y = (ary[4] << 8) | ary[5];
	c.x = x * CMPS_SCALE;
	c.y = y * CMPS_SCALE;
	c.z = z * CMPS_SCALE;
}
// ＠XY軸
void GY80::getCompassAxisXY(){
	unsigned char ary[6];
	wire_readArray(ary, IIC_ADDR_CMPS, CMPS_DATA_XMSB, 6);
	short x, y;
	x = (ary[0] << 8) | ary[1];
	y = (ary[4] << 8) | ary[5]; //レジスタの並びがX, Z, Yであることに注意
	c.x = x * CMPS_SCALE;
	c.y = y * CMPS_SCALE;
}
// ＠XZ軸
void GY80::getCompassAxisXZ(){
	unsigned char ary[6];
	wire_readArray(ary, IIC_ADDR_CMPS, CMPS_DATA_XMSB, 6);
	short x, z;
	x = (ary[0] << 8) | ary[1];
	z = (ary[2] << 8) | ary[3]; //レジスタの並びがX, Z, Yであることに注意
	c.x = x * CMPS_SCALE;
	c.z = z * CMPS_SCALE;
}
// ＠YZ軸
void GY80::getCompassAxisYZ(){
	unsigned char ary[6];
	wire_readArray(ary, IIC_ADDR_CMPS, CMPS_DATA_XMSB, 6);
	short y, z;
	y = (ary[4] << 8) | ary[5]; //レジスタの並びがX, Z, Yであることに注意
	z = (ary[2] << 8) | ary[3];
	c.y = y * CMPS_SCALE;
	c.z = z * CMPS_SCALE;
}
// ＠X軸
void GY80::getCompassAxisX(){
	short x = (wire_read(IIC_ADDR_CMPS, CMPS_DATA_XMSB) << 8) | wire_read(IIC_ADDR_CMPS, CMPS_DATA_XLSB);
	c.x = x * CMPS_SCALE;
}
// ＠Y軸
void GY80::getCompassAxisY(){
	short y = (wire_read(IIC_ADDR_CMPS, CMPS_DATA_YMSB) << 8) | wire_read(IIC_ADDR_CMPS, CMPS_DATA_YLSB);
	c.y = y * CMPS_SCALE;
}
// ＠Z軸
void GY80::getCompassAxisZ(){
	short z = (wire_read(IIC_ADDR_CMPS, CMPS_DATA_ZMSB) << 8) | wire_read(IIC_ADDR_CMPS, CMPS_DATA_ZLSB);
	c.z = z * CMPS_SCALE;
}

// 気圧センサー初期化 ===============================================
void GY80::initAPressure(){
	delay(20);
	wire_read(IIC_ADDR_APRESS, APRESS_MD_H); //dummy(最初のほうの信号だけ妙に安定しないため)
	wire_read(IIC_ADDR_APRESS, APRESS_MD_L); //dummy(最初のほうの信号だけ妙に安定しないため)
	delay(20);
	wire_read(IIC_ADDR_APRESS, APRESS_MD_H); //dummy(最初のほうの信号だけ妙に安定しないため)
	wire_read(IIC_ADDR_APRESS, APRESS_MD_L); //dummy(最初のほうの信号だけ妙に安定しないため)
	delay(50);

	//計算用補正データの取得
	ap.a1 = (wire_read(IIC_ADDR_APRESS, APRESS_A1_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_A1_L);
	ap.a2 = (wire_read(IIC_ADDR_APRESS, APRESS_A2_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_A2_L);
	ap.a3 = (wire_read(IIC_ADDR_APRESS, APRESS_A3_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_A3_L);
	ap.a4 = (wire_read(IIC_ADDR_APRESS, APRESS_A4_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_A4_L);
	ap.a5 = (wire_read(IIC_ADDR_APRESS, APRESS_A5_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_A5_L);
	ap.a6 = (wire_read(IIC_ADDR_APRESS, APRESS_A6_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_A6_L);
	
	ap.b1 = (wire_read(IIC_ADDR_APRESS, APRESS_B1_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_B1_L);
	ap.b2 = (wire_read(IIC_ADDR_APRESS, APRESS_B2_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_B2_L);
	
	ap.mb = (wire_read(IIC_ADDR_APRESS, APRESS_MB_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_MB_L);
	ap.mc = (wire_read(IIC_ADDR_APRESS, APRESS_MC_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_MC_L);
	ap.md = (wire_read(IIC_ADDR_APRESS, APRESS_MD_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_MD_L);
	
	delay(300);
}

// 計算用補正データの更新(ふつうは呼ばなくていいハズ) ===============
void GY80::updateValAPressure(){
	delay(20);

	//計算用補正データの取得
	ap.a1 = (wire_read(IIC_ADDR_APRESS, APRESS_A1_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_A1_L);
	ap.a2 = (wire_read(IIC_ADDR_APRESS, APRESS_A2_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_A2_L);
	ap.a3 = (wire_read(IIC_ADDR_APRESS, APRESS_A3_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_A3_L);
	ap.a4 = (wire_read(IIC_ADDR_APRESS, APRESS_A4_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_A4_L);
	ap.a5 = (wire_read(IIC_ADDR_APRESS, APRESS_A5_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_A5_L);
	ap.a6 = (wire_read(IIC_ADDR_APRESS, APRESS_A6_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_A6_L);
	
	ap.b1 = (wire_read(IIC_ADDR_APRESS, APRESS_B1_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_B1_L);
	ap.b2 = (wire_read(IIC_ADDR_APRESS, APRESS_B2_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_B2_L);
	
	ap.mb = (wire_read(IIC_ADDR_APRESS, APRESS_MB_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_MB_L);
	ap.mc = (wire_read(IIC_ADDR_APRESS, APRESS_MC_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_MC_L);
	ap.md = (wire_read(IIC_ADDR_APRESS, APRESS_MD_H) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_MD_L);
	
	delay(100);
}

// 気圧センサーデータ取得 ============================================
// ＠気温計算用数値UT
long GY80::getAPressureTempParam(){
	wire_write(IIC_ADDR_APRESS, APRESS_REG, APRESS_TEMPERATURE);
	delay(5);
	return (wire_read(IIC_ADDR_APRESS, APRESS_MSB) << 8) | wire_read(IIC_ADDR_APRESS, APRESS_LSB);
}
// ＠気圧計算用数値UP
long GY80::getAPressurePressParam(unsigned char resolution){
	resolution = (resolution > 3)? 3 : resolution;
	
	switch(resolution){
		case 0:
			wire_write(IIC_ADDR_APRESS, APRESS_REG, APRESS_PRESSURE_ULP);
			delay(5);
			break;
		case 1:
			wire_write(IIC_ADDR_APRESS, APRESS_REG, APRESS_PRESSURE_STD);
			delay(8);
			break;
		case 2:
			wire_write(IIC_ADDR_APRESS, APRESS_REG, APRESS_PRESSURE_HR);
			delay(14);
			break;
		case 3:
			wire_write(IIC_ADDR_APRESS, APRESS_REG, APRESS_PRESSURE_UHR);
			delay(26);
			break;
	}
	
	long retval =
		(wire_read(IIC_ADDR_APRESS, APRESS_MSB) << 16) |
		(wire_read(IIC_ADDR_APRESS, APRESS_LSB) << 8) |
		wire_read(IIC_ADDR_APRESS, APRESS_XLSB);
	
	return retval >> (8 - resolution);
}
// ＠気温計算用数値UTを取得して計算して値に格納(単位 C:セルシウス度)
void GY80::getAPressureCalcTemp(){
	ap.temp = calcTemp(getAPressureTempParam());
}
// ＠気圧計算用数値UPを取得して計算して値に格納(単位 Pa:パスカル)
void GY80::getAPressureCalcPress(unsigned char resolution){
	ap.pressure = calcPressure(getAPressurePressParam(resolution), resolution);
}
// ＠気温計算用数値UTから気温を算出(単位 C:セルシウス度)
float GY80::calcTemp(long tempParam){
	//以下計算式はデータシートより
	long x1 = ((tempParam - ap.a6) * ap.a5) >> 15;
	ap.b5 = x1 + ((ap.mc << 11) / (x1 + ap.md));
	float retval = (float)(ap.b5 + 8) / 16;
	return retval / 10;
}
// ＠気圧計算用数値UPから気圧を算出(単位 Pa:パスカル)
long GY80::calcPressure(long pressParam, unsigned char resolution){
	resolution = (resolution > 3)? 3 : resolution;
	//以下計算式はデータシートより
	long b6 = ap.b5 - 4000;
	long bb6 = (b6 * b6) >> 12;
	long b3 = ((((ap.a1 << 2) + (((ap.b2 * bb6) >> 11) + ((ap.a2 * b6) >> 11))) << resolution) + 2) >> 2;
	unsigned long b4 = (ap.a4 * (unsigned long)(((((ap.a3 * b6 >> 13) + ((ap.b1 * bb6) >> 16)) + 2) >> 2)+32768)) >> 15;
	unsigned long b7 = ((unsigned long)pressParam - b3) * (50000 >> resolution);
	long p = (b7 < 0x80000000)? (b7 << 1) / b4: (b7 / b4) << 1;
	long x1 = ((p >> 8) * (p >> 8) * 3038) >> 16;
	long x2 = (-7357 * p) >> 16;
	return p + ((x1 + x2 + 3791) >> 4);
}
// ＠気圧(Pa:パスカル)を大気圧に換算
float GY80::calcPa2Atm(long pressurePa_value){
	return pressurePa_value / 101325;
}
// ＠海面気圧、現地気圧、現地気温から標高を算出
float GY80::calcAltitudeSample(long sealevelPa_value, long nowPa_value, float nowTemp){
	float sea_hPa = sealevelPa_value / 100;
	float now_hPa = nowPa_value / 100;
	return ((pow(sea_hPa / now_hPa, 0.190222560395662) - 1) * (nowTemp + 273.15)) / 0.0065;
}

// ARHS用RAWデータ取得 ==============================================
// ＠6軸
void GY80::getRaw6(short &gx, short &gy, short &gz, short &ax, short &ay, short &az){
	unsigned char ary[6];
	gx = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_L);
	gy = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_L);
	gz = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_L);
	wire_readArray(ary, IIC_ADDR_ACCEL, ACCEL_DATAX0, 6);
	ax = (ary[1] << 8) | ary[0];
	ay = (ary[3] << 8) | ary[2];
	az = (ary[5] << 8) | ary[4];
}
// ＠9軸
void GY80::getRaw9(short &gx, short &gy, short &gz, short &ax, short &ay, short &az, short &mx, short &my, short &mz){
	unsigned char ary[6];
	gx = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_X_L);
	gy = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Y_L);
	gz = (wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_H) << 8) | wire_read(IIC_ADDR_GYRO, GYRO_OUT_Z_L);
	wire_readArray(ary, IIC_ADDR_ACCEL, ACCEL_DATAX0, 6);
	ax = (ary[1] << 8) | ary[0];
	ay = (ary[3] << 8) | ary[2];
	az = (ary[5] << 8) | ary[4];
	wire_readArray(ary, IIC_ADDR_CMPS, CMPS_DATA_XMSB, 6);
	mx = (ary[0] << 8) | ary[1];
	my = (ary[4] << 8) | ary[5]; //レジスタの並びがX, Z, Yであることに注意
	mz = (ary[2] << 8) | ary[3];
}