#include <Arduino.h>
#include <Wire.h>

#ifndef GY80_LIB
#define GY80_LIB

// [使用環境に合わせて書き換えてください]============================

// I2C(TwoWire) 使用チャンネル
// GR-SAKURA、GR-KAEDEで使えるものはWire, Wire1, Wire2, Wire3, Wire4, Wire5, Wire6, Wire7
// GR-KURUMIはWireのみ
#define IIC_CHANNEL_NAME Wire
// I2C周波数 (参考:NORMAL=100KHz, HIGHSPEED=200KHz)
#define IIC_SIGNALFREQ_HZ 200000

// 以下、変更不要 ===================================================

#define IIC_ADDR_GYRO 0x69 //L3G4200D
#define IIC_ADDR_ACCEL 0x53 //ADXL345
#define IIC_ADDR_CMPS 0x1E //HMC5883L
#define IIC_ADDR_APRESS 0x77 //BMP085

//L3G4200D(GYROSCOPE)
#define GYRO_SCALE 0.0175
#define GYRO_CTRL_REG1 0x20
#define GYRO_CTRL_REG2 0x21
#define GYRO_CTRL_REG3 0x22
#define GYRO_CTRL_REG4 0x23
#define GYRO_CTRL_REG5 0x24
#define GYRO_REFERENCE 0x25
#define GYRO_OUT_TEMP 0x26
#define GYRO_STATUS_REG 0x27
#define GYRO_OUT_X_L 0x28
#define GYRO_OUT_X_H 0x29
#define GYRO_OUT_Y_L 0x2A
#define GYRO_OUT_Y_H 0x2B
#define GYRO_OUT_Z_L 0x2C
#define GYRO_OUT_Z_H 0x2D
#define GYRO_FIFO_CTRL_REG 0x2E
#define GYRO_FIFO_SRC_REG 0x2F
#define GYRO_INT1_CFG 0x30
#define GYRO_INT1_SRC 0x31
#define GYRO_INT1_TSH_XH 0x32
#define GYRO_INT1_TSH_XL 0x33
#define GYRO_INT1_TSH_YH 0x34
#define GYRO_INT1_TSH_YL 0x35
#define GYRO_INT1_TSH_ZH 0x36
#define GYRO_INT1_TSH_ZL 0x37
#define GYRO_INT1_DURATION 0x38

//ADXL345(ACCELEROMETER)
#define ACCEL_SCALE 0.00390625
#define ACCEL_THRESH_TAP 0x1D
#define ACCEL_OFSX 0x1E
#define ACCEL_OFSY 0x1F
#define ACCEL_OFSZ 0x20
#define ACCEL_DUR 0x21
#define ACCEL_Latent 0x22
#define ACCEL_Window 0x23
#define ACCEL_THRESH_ACT 0x24
#define ACCEL_THRESH_INACT 0x25
#define ACCEL_TIME_INACT 0x26
#define ACCEL_ACT_INACT_CTL 0x27
#define ACCEL_THRESH_FF 0x28
#define ACCEL_TIME_FF 0x29
#define ACCEL_TAP_AXES 0x2A
#define ACCEL_ACT_TAP_STATUS 0x2B
#define ACCEL_BW_RATE 0x2C
#define ACCEL_POWER_CTL 0x2D
#define ACCEL_INT_ENABLE 0x2E
#define ACCEL_INT_MAP 0x2F
#define ACCEL_INT_SOURCE 0x30
#define ACCEL_DATA_FORMAT 0x31
#define ACCEL_DATAX0 0x32
#define ACCEL_DATAX1 0x33
#define ACCEL_DATAY0 0x34
#define ACCEL_DATAY1 0x35
#define ACCEL_DATAZ0 0x36
#define ACCEL_DATAZ1 0x37
#define ACCEL_FIFO_CTL 0x38
#define ACCEL_FIFO_STATUS 0x39

//HMC5883L(COMPASS)
#define CMPS_SCALE 0.00122
#define CMPS_REGA 0x00
#define CMPS_REGB 0x01
#define CMPS_MODE 0x02
#define CMPS_DATA_XMSB 0x03
#define CMPS_DATA_XLSB 0x04
#define CMPS_DATA_ZMSB 0x05
#define CMPS_DATA_ZLSB 0x06
#define CMPS_DATA_YMSB 0x07
#define CMPS_DATA_YLSB 0x08
#define CMPS_STATUS 0x09

//BMP085(ATMOSPHERIC PRESSURE)
#define APRESS_REG 0xF4
#define APRESS_TEMPERATURE 0x2E
#define APRESS_PRESSURE_ULP 0x34
#define APRESS_PRESSURE_STD 0x74
#define APRESS_PRESSURE_HR 0xB4
#define APRESS_PRESSURE_UHR 0xF4
#define APRESS_MSB 0xF6
#define APRESS_LSB 0xF7
#define APRESS_XLSB 0xF8
#define APRESS_A1_H 0xAA
#define APRESS_A1_L 0xAB
#define APRESS_A2_H 0xAC
#define APRESS_A2_L 0xAD
#define APRESS_A3_H 0xAE
#define APRESS_A3_L 0xAF
#define APRESS_A4_H 0xB0
#define APRESS_A4_L 0xB1
#define APRESS_A5_H 0xB2
#define APRESS_A5_L 0xB3
#define APRESS_A6_H 0xB4
#define APRESS_A6_L 0xB5
#define APRESS_B1_H 0xB6
#define APRESS_B1_L 0xB7
#define APRESS_B2_H 0xB8
#define APRESS_B2_L 0xB9
#define APRESS_MB_H 0xBA
#define APRESS_MB_L 0xBB
#define APRESS_MC_H 0xBC
#define APRESS_MC_L 0xBD
#define APRESS_MD_H 0xBE
#define APRESS_MD_L 0xBF

//ジャイロセンサー用構造体
struct s_gyro{
	float x;
	float y;
	float z;
	short temp;
};

//加速度センサー用構造体
struct s_accel{
	float x;
	float y;
	float z;
	float offX;
	float offY;
	float offZ;
};

//地磁気センサー用構造体
struct s_cmps{
	float x;
	float y;
	float z;
};

//気圧センサー用構造体
struct s_apress{
	short a1;
	short a2;
	short a3;
	unsigned short a4;
	unsigned short a5;
	unsigned short a6;
	short b1;
	short b2;
	long b5;
	short mb;
	short mc;
	short md;
	float temp;
	long pressure;
};

//GY80管理クラス ====================================================
class GY80{
public:
	GY80(char modeflag=0x0F);
	
private:
	char umode;
	
	//I2C読み取り用
	unsigned char wire_read(unsigned char devAddr, unsigned char reg);
	void wire_readArray(unsigned char *buff, unsigned char devAddr, unsigned char reg, unsigned char length);
	
	//I2C書き込み用
	void wire_write(unsigned char devAddr, unsigned char reg, unsigned char val);

public:
	//各センサーからの値管理用構造体
	s_gyro g; //ジャイロ用
	s_accel ac; //加速度用
	s_cmps c; //コンパス用
	s_apress ap; //気圧用
	
	//I2Cとセンサーの初期化開始
	void init(bool i2cinit=true);
	
	//L3G4200D(GYROSCOPE)
	void initGyro(); //初期化用(通常はinitで勝手に呼ばれるので明示的に呼び出す必要はない)
	void getGyroAll(bool isDEG=true); //全軸の角速度と温度を取得して構造体gの値を更新
	void getGyroAxis(bool isDEG=true); //全軸の角速度を取得して構造体gの値を更新
	void getGyroAxisXY(bool isDEG=true); //XY軸の角速度を取得して構造体gの値を更新
	void getGyroAxisXZ(bool isDEG=true); //XZ軸の角速度を取得して構造体gの値を更新
	void getGyroAxisYZ(bool isDEG=true); //YZ軸の角速度を取得して構造体gの値を更新
	void getGyroAxisX(bool isDEG=true); //X軸の角速度を取得して構造体gの値を更新
	void getGyroAxisY(bool isDEG=true); //Y軸の角速度を取得して構造体gの値を更新
	void getGyroAxisZ(bool isDEG=true); //Z軸の角速度を取得して構造体gの値を更新
	void getGyroTemp(); //温度を取得して構造体gの値を更新
	
	//ADXL345(ACCELEROMETER)
	void initAccelero(); //初期化用(通常はinitで勝手に呼ばれるので明示的に呼び出す必要はない)
	void setAcceleroOffsetAuto(char sample_n=20); //加速度センサーの自動補正(sample_nで指定された回数の取得平均を補正値としてレジスタに書き込む)
	void setAcceleroOffsetAuto(bool targetX, bool targetY, bool targetZ, char sample_n=20); //加速度センサーの自動補正(対象軸指定)
	void setAcceleroOffset(float offsetX, float offsetY, float offsetZ); //補正値手動設定
	void getAcceleroAll(); //全軸の加速度と補正値を取得して構造体acの値を更新
	void getAcceleroAxis(); //全軸の加速度を取得して構造体acの値を更新
	void getAcceleroAxisXY(); //XY軸の加速度を取得して構造体acの値を更新
	void getAcceleroAxisXZ(); //XZ軸の加速度を取得して構造体acの値を更新
	void getAcceleroAxisYZ(); //YZ軸の加速度を取得して構造体acの値を更新
	void getAcceleroAxisX(); //X軸の加速度を取得して構造体acの値を更新
	void getAcceleroAxisY(); //Y軸の加速度を取得して構造体acの値を更新
	void getAcceleroAxisZ(); //Z軸の加速度を取得して構造体acの値を更新
	void getAcceleroOffset(); //全軸の補正値を取得して構造体acの値を更新
	
	//HMC5883L(COMPASS)
	void initCompass(); //初期化用(通常はinitで勝手に呼ばれるので明示的に呼び出す必要はない)
	void getCompassAll(); //全軸の地磁気の値を取得して構造体cの値を更新
	void getCompassAxis(); //getCompassAllと同一
	void getCompassAxisXY(); //XY軸の地磁気の値を取得して構造体cの値を更新
	void getCompassAxisXZ(); //XZ軸の地磁気の値を取得して構造体cの値を更新
	void getCompassAxisYZ(); //YZ軸の地磁気の値を取得して構造体cの値を更新
	void getCompassAxisX(); //X軸の地磁気の値を取得して構造体cの値を更新
	void getCompassAxisY(); //Y軸の地磁気の値を取得して構造体cの値を更新
	void getCompassAxisZ(); //Z軸の地磁気の値を取得して構造体cの値を更新
	
	//BMP085(ATMOSPHERIC PRESSURE)
	void initAPressure(); //初期化用(通常はinitで勝手に呼ばれるので明示的に呼び出す必要はない)
	void updateValAPressure(); //数値計算用係数を再ロード
	long getAPressureTempParam(); //気温算出用の値を取得
	long getAPressurePressParam(unsigned char resolution=1); //気圧算出用値を取得(resolution:精度 1-3[3が最高精度])
	void getAPressureCalcTemp(); //気温算出用の値から気温を計算して構造体apの値を更新
	void getAPressureCalcPress(unsigned char resolution=1); //気圧算出用の値から気圧を計算して構造体apの値を更新(resolution:精度 1-3[3が最高精度])
	float calcTemp(long tempParam); //気温算出用の値から気温を計算
	long calcPressure(long pressParam, unsigned char resolution=1); //気圧算出用の値から気圧を計算
	float calcPa2Atm(long pressurePa_value); //気圧(パスカル)を大気圧値に変換
	float calcAltitudeSample(long sealevelPa_value, long nowPa_value, float nowTemp); //海面基準気圧から現在の標高を算出
	
	//ARHS用RAWデータ出力
	void getRaw6(short &gx, short &gy, short &gz, short &ax, short &ay, short &az);
	void getRaw9(short &gx, short &gy, short &gz, short &ax, short &ay, short &az, short &mx, short &my, short &mz);
};

#endif

/* change log *****
2015/11/18:
・加速度の補正が常に0になる問題を修正
・加速度の初期化時の自動補正を削除
・setAcceleroOffsetAuto(bool targetX, bool targetY, bool targetZ, char sample_n)を追加
2015/11/19:
・補正値を微調整
・getRaw6(short &gx, short &gy, short &gz, short &ax, short &ay, short &az)を追加
・getRaw9(short &gx, short &gy, short &gz, short &ax, short &ay, short &az, short &mx, short &my, short &mz)を追加
・コンパスの出力値をmGa(ミリガウス)に変更
2015/11/22:
・多数の不具合を修正
・コンパスの出力値をGa(ガウス)に変更
*/