#include <Arduino.h>
#include "GY80.h"

//GY80インスタンス:すべてのセンサーを使用
//I2CはWire1を使用(GY80.h内のIIC_CHANNEL_NAMEで定義)
GY80 sensor(0x0F);

//Serial表示用
char msg[200];

//標高計算用
float wk;

//セットアップ:最初の1回だけ実行される
void setup(){
	pinMode(PIN_LED0,OUTPUT); //LED0
	pinMode(PIN_LED2,OUTPUT); //LED2
	
	pinMode(PIN_SW,INPUT); //押しボタン(青)
	
	Serial.begin(9600); //シリアル通信開始準備
	
	digitalWrite(PIN_LED0, HIGH); //LED0点灯
	digitalWrite(PIN_LED2, HIGH); //LED2点灯
	
	while(digitalRead(PIN_SW) == 1); //押しボタン(青)が押されるまで待ち続ける
	
	digitalWrite(PIN_LED2, LOW); //LED2消灯
	sensor.init(); //センサー初期化
}

//繰り返し無限に実行される
void loop(){
	
	//ジャイロセンサーのすべての値を更新
	sensor.getGyroAll();
	
	//加速度センサーのすべての軸の値を更新
	sensor.getAcceleroAxis();
	
	//地磁気センサーのすべての軸の値を更新
	sensor.getCompassAxis();
	
	//気圧センサーから気温を算出し、セルシウス度(C)で取得
	sensor.getAPressureCalcTemp();
	
	//気圧センサーから気圧を最高精度(3)で算出し、パスカル(Pa)で取得
	sensor.getAPressureCalcPress(3);
	
	//海面気圧を1024 hPa(102400 Pa)として現在地の標高を計算し、メートル(m)で取得し変数wkに格納
	wk = sensor.calcAltitudeSample(102400, sensor.ap.pressure, sensor.ap.temp);
	
	
	//ジャイロセンサーの値を表示(変数msgにsprintfで表示文字列を格納し、Serial.printlnで表示)
	sprintf(
		msg,
		"[Gyro] gT:%03d[degC], gX:%04.5f[dps], gY:%04.5f[dps], gZ:%04.5f[dps]",
		sensor.g.temp, sensor.g.x, sensor.g.y, sensor.g.z
	);
	
	Serial.println(msg);
	
	//加速度センサーの値を表示(変数msgにsprintfで表示文字列を格納し、Serial.printlnで表示)
	sprintf(
		msg,
		"[Accelero] acX:%03.5f[g], acY:%03.5f[g], acZ:%03.5f[g], offX:%03.5f[g], offY:%03.5f[g], offZ:%03.5f[g]",
		sensor.ac.x, sensor.ac.y, sensor.ac.z, sensor.ac.offX, sensor.ac.offY, sensor.ac.offZ
	);
	
	Serial.println(msg);
	
	//地磁気センサーの値を表示(変数msgにsprintfで表示文字列を格納し、Serial.printlnで表示)
	sprintf(
		msg,
		"[Compass] cX:%03.5f[Ga], cY:%03.5f[Ga], cZ:%03.5f[Ga]",
		sensor.c.x, sensor.c.y, sensor.c.z
	);
	
	Serial.println(msg);
	
	//気温、気圧、算出標高を表示(変数msgにsprintfで表示文字列を格納し、Serial.printlnで表示)
	sprintf(
		msg,
		"[A Pressure] apTemp:%3.5f[C], apPressure:%08ld[Pa], Sample Hight:%5.5f[m]",
		sensor.ap.temp, sensor.ap.pressure, wk
	);
	
	Serial.println(msg);
	
	/*
	//ARHS用：ジャイロ、加速度、コンパスのRAWデータを取得
	short gx, gy, gz, ax, ay, az, mx, my, mz;
	sensor.getRaw9(gx, gy, gz, ax, ay, az, mx, my, mz);
	sprintf(msg, "%d,%d,%d:%d,%d,%d:%d,%d,%d", gx, gy, gz, ax, ay, az, mx, my, mz);
	Serial.println(msg);
	*/
	
	//50ミリ秒待つ
	delay(50);
	
	//押しボタン(青)が押されていたらプログラム実行を終了
	if(digitalRead(PIN_SW) == 0){
		digitalWrite(PIN_LED0, LOW); //LED0消灯
		exit(1);
	}
}